"use strict";
/**
 * Copyright 2019 Google LLC. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const semver = require("semver");
const checkpoint_1 = require("./checkpoint");
const conventional_commits_1 = require("./conventional-commits");
const github_1 = require("./github");
const release_pr_1 = require("./release-pr");
const release_pr_2 = require("./release-pr");
const parseGithubRepoUrl = require('parse-github-repo-url');
const ISSUE_TITLE = 'chore(release): proposal for next release';
const ISSUE_FOOTER = '[//]: # footer follows.';
const CHECKBOX = '* [ ] **Should I create this release for you :robot:?**';
const CHECK_REGEX = /\[x]/;
class CandidateIssue {
    constructor(options) {
        this.bumpMinorPreMajor = options.bumpMinorPreMajor || false;
        this.label = options.label;
        this.repoUrl = options.repoUrl;
        this.token = options.token;
        this.packageName = options.packageName;
        this.releaseType = options.releaseType;
        this.gh = this.gitHubInstance();
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            switch (this.releaseType) {
                case release_pr_1.ReleaseType.Node:
                    yield this.nodeReleaseCandidate();
                    break;
                default:
                    throw Error('unknown release type');
            }
        });
    }
    nodeReleaseCandidate() {
        return __awaiter(this, void 0, void 0, function* () {
            const latestTag = yield this.gh.latestTag();
            const commits = yield this.commits(latestTag ? latestTag.sha : undefined);
            const cc = new conventional_commits_1.ConventionalCommits({
                commits,
                githubRepoUrl: this.repoUrl,
                bumpMinorPreMajor: this.bumpMinorPreMajor
            });
            const candidate = yield this.coerceReleaseCandidate(cc, latestTag);
            const changelogEntry = yield cc.generateChangelogEntry({
                version: candidate.version,
                currentTag: `v${candidate.version}`,
                previousTag: candidate.previousTag
            });
            const issue = yield this.gh.findExistingReleaseIssue(ISSUE_TITLE);
            let body = CandidateIssue.bodyTemplate(changelogEntry, this.packageName);
            if (issue) {
                if (CHECK_REGEX.test(issue.body)) {
                    // if the checkox has been clicked for a release
                    // mint the release.
                    checkpoint_1.checkpoint(`release checkbox was checked, creating release`, checkpoint_1.CheckpointType.Success);
                    const rp = new release_pr_2.ReleasePR({
                        bumpMinorPreMajor: this.bumpMinorPreMajor,
                        label: this.label,
                        token: this.token,
                        repoUrl: this.repoUrl,
                        packageName: this.packageName,
                        releaseType: this.releaseType
                    });
                    const prNumber = yield rp.run();
                    body = body.replace(CHECKBOX, `**release created at #${prNumber}**`);
                }
                else if (CandidateIssue.bodySansFooter(issue.body) === CandidateIssue.bodySansFooter(body)) {
                    // don't update the issue if the content is the same for the release.
                    checkpoint_1.checkpoint(`skipping update to #${issue.number}, no change to body`, checkpoint_1.CheckpointType.Failure);
                    return;
                }
            }
            yield this.gh.openIssue(ISSUE_TITLE, body, issue);
        });
    }
    coerceReleaseCandidate(cc, latestTag) {
        return __awaiter(this, void 0, void 0, function* () {
            const previousTag = latestTag ? latestTag.name : undefined;
            let version = latestTag ? latestTag.version : '1.0.0';
            if (latestTag) {
                const bump = yield cc.suggestBump(version);
                const candidate = semver.inc(version, bump.releaseType);
                if (!candidate)
                    throw Error(`failed to increment ${version}`);
                version = candidate;
            }
            return { version, previousTag };
        });
    }
    commits(sha) {
        return __awaiter(this, void 0, void 0, function* () {
            const commits = yield this.gh.commitsSinceSha(sha);
            if (commits.length) {
                checkpoint_1.checkpoint(`found ${commits.length} commits since ${sha}`, checkpoint_1.CheckpointType.Success);
            }
            else {
                checkpoint_1.checkpoint(`no commits found since ${sha}`, checkpoint_1.CheckpointType.Failure);
            }
            return commits;
        });
    }
    gitHubInstance() {
        const [owner, repo] = parseGithubRepoUrl(this.repoUrl);
        return new github_1.GitHub({ token: this.token, owner, repo });
    }
    static bodyTemplate(changelogEntry, packageName) {
        return `_:robot: Here's what the next release of **${packageName}** would look like._

---

${changelogEntry}

---

${ISSUE_FOOTER}

${CHECKBOX}
`;
    }
    static bodySansFooter(body) {
        const footerPosition = body.indexOf(ISSUE_FOOTER);
        if (footerPosition === -1)
            return body;
        else
            return body.slice(0, footerPosition);
    }
}
exports.CandidateIssue = CandidateIssue;
//# sourceMappingURL=candidate-issue.js.map