"use strict";
/**
 * Copyright 2019 Google LLC. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const semver = require("semver");
const checkpoint_1 = require("./checkpoint");
const conventional_commits_1 = require("./conventional-commits");
const github_1 = require("./github");
const changelog_1 = require("./updaters/changelog");
const package_json_1 = require("./updaters/package-json");
const samples_package_json_1 = require("./updaters/samples-package-json");
const parseGithubRepoUrl = require('parse-github-repo-url');
var ReleaseType;
(function (ReleaseType) {
    ReleaseType["Node"] = "node";
})(ReleaseType = exports.ReleaseType || (exports.ReleaseType = {}));
class ReleasePR {
    constructor(options) {
        this.bumpMinorPreMajor = options.bumpMinorPreMajor || false;
        this.label = options.label;
        this.repoUrl = options.repoUrl;
        this.token = options.token;
        this.packageName = options.packageName;
        this.releaseAs = options.releaseAs;
        this.releaseType = options.releaseType;
        this.gh = this.gitHubInstance();
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            switch (this.releaseType) {
                case ReleaseType.Node:
                    return yield this.nodeRelease();
                default:
                    throw Error('unknown release type');
            }
        });
    }
    nodeRelease() {
        return __awaiter(this, void 0, void 0, function* () {
            const latestTag = yield this.gh.latestTag();
            const commits = yield this.commits(latestTag ? latestTag.sha : undefined);
            const cc = new conventional_commits_1.ConventionalCommits({
                commits,
                githubRepoUrl: this.repoUrl,
                bumpMinorPreMajor: this.bumpMinorPreMajor
            });
            const candidate = yield this.coerceReleaseCandidate(cc, latestTag);
            const changelogEntry = yield cc.generateChangelogEntry({
                version: candidate.version,
                currentTag: `v${candidate.version}`,
                previousTag: candidate.previousTag
            });
            const updates = [];
            updates.push(new changelog_1.Changelog({
                path: 'CHANGELOG.md',
                changelogEntry,
                version: candidate.version,
                packageName: this.packageName
            }));
            updates.push(new package_json_1.PackageJson({
                path: 'package.json',
                changelogEntry,
                version: candidate.version,
                packageName: this.packageName
            }));
            updates.push(new samples_package_json_1.SamplesPackageJson({
                path: 'samples/package.json',
                changelogEntry,
                version: candidate.version,
                packageName: this.packageName
            }));
            const sha = this.shaFromCommits(commits);
            const title = `chore: release ${candidate.version}`;
            const body = `:robot: I have created a release \\*beep\\* \\*boop\\* \n---\n${changelogEntry}`;
            const pr = yield this.gh.openPR({
                branch: `release-v${candidate.version}`,
                version: candidate.version,
                sha,
                updates,
                title,
                body
            });
            yield this.gh.addLabel(pr, this.label);
            return pr;
        });
    }
    coerceReleaseCandidate(cc, latestTag) {
        return __awaiter(this, void 0, void 0, function* () {
            const previousTag = latestTag ? latestTag.name : undefined;
            let version = latestTag ? latestTag.version : '1.0.0';
            if (latestTag && !this.releaseAs) {
                const bump = yield cc.suggestBump(version);
                const candidate = semver.inc(version, bump.releaseType);
                if (!candidate)
                    throw Error(`failed to increment ${version}`);
                version = candidate;
            }
            else if (this.releaseAs) {
                version = this.releaseAs;
            }
            return { version, previousTag };
        });
    }
    commits(sha) {
        return __awaiter(this, void 0, void 0, function* () {
            const commits = yield this.gh.commitsSinceSha(sha);
            if (commits.length) {
                checkpoint_1.checkpoint(`found ${commits.length} commits since ${sha}`, checkpoint_1.CheckpointType.Success);
            }
            else {
                checkpoint_1.checkpoint(`no commits found since ${sha}`, checkpoint_1.CheckpointType.Failure);
            }
            return commits;
        });
    }
    gitHubInstance() {
        const [owner, repo] = parseGithubRepoUrl(this.repoUrl);
        return new github_1.GitHub({ token: this.token, owner, repo });
    }
    shaFromCommits(commits) {
        // The conventional commits parser expects an array of string commit
        // messages terminated by `-hash-` followed by the commit sha. We
        // piggyback off of this, and use this sha when choosing a
        // point to branch from for PRs.
        const split = commits[0].split('-hash-');
        return split[split.length - 1].trim();
    }
}
exports.ReleasePR = ReleasePR;
//# sourceMappingURL=release-pr.js.map